@@ -3,6 +3,7 @@ package testjson
 import (
 	"bytes"
 	"math/rand"
+	"os"
 	"runtime"
 	"testing"
 	"testing/quick"
@@ -19,6 +20,7 @@ import (
 
 func TestScanTestOutput_WithDotsFormatter(t *testing.T) {
 	skip.If(t, runtime.GOOS == "windows")
+	skip.If(t, os.Getenv("CI") == "true", "flaky on Github Actions")
 
 	out := new(bytes.Buffer)
 	dotfmt := &dotFormatter{
@@ -129,7 +131,7 @@ func TestFmtDotElapsed_RuneCountProperty(t *testing.T) {
 
 func TestNewDotFormatter(t *testing.T) {
 	buf := new(bytes.Buffer)
-	ef := newDotFormatter(buf)
+	ef := newDotFormatter(buf, FormatOptions{})
 
 	d, ok := ef.(*dotFormatter)
 	skip.If(t, !ok, "no terminal width")
